#include <stdio.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "tmpasm_scconfig.h"
#include "../libvmma/scconfig_hooks.h"

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if (strcmp(key, "debug") == 0) {
		vmma_set_debug(strue);
		return 1;
	}

	if (strcmp(key, "prefix") == 0) {
		vmma_set_prefix(value);
		return 1;
	}

	return 0;
}


/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	vmma_hook_postinit();
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	return vmma_hook_detect_host();
}

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	require("cc/cc", 0, 0);
	require("cc/cflags", 0, 0);

	return vmma_hook_detect_target();
}

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	const char* const root = "../test-src";
	int generr = 0;

	generr |= vmma_hook_generate("../libvmma");

	fprintf(stderr, "Generating doc/Makefile (%d)\n", generr |= tmpasm("../doc", "Makefile.in", "Makefile"));
	fprintf(stderr, "Generating test-src/Makefile (%d)\n", generr |= tmpasm(root, "Makefile.in", "Makefile"));
	fprintf(stderr, "Generating test-src/config.h (%d)\n", generr |= tmpasm(root, "config.h.in", "config.h"));

	return generr;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

