#include "config.h"
#include "test.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <assert.h>
#include "libvmma/buddy.h"
#include "shuffle.h"

#define DISPLAY_TITLE       0
#define DISPLAY_ALLOCATIONS 0

static uint8_t memory[64*1024];
static uint8_t memory2[2*sizeof(memory)];
static int alloc_state;

static void* eb_alloc(void* ctx, size_t size)
{
	if(0<alloc_state) { return NULL; }
	if(size>sizeof(memory2)) { return NULL; }
	++alloc_state;
	return memory2;
}

static void eb_free(void* ctx, void* mem)
{
	if(0<alloc_state && mem==(void*)memory2)
	{
		alloc_state = 0;
	}
}

static void shuffle(void** array, size_t nelem)
{
	shuffle_pointers(array, nelem);
}

static struct buddy* create_buddy(const int use_ebuddy, const int heap_size_percent)
{
	const size_t minsize = 32;
	const size_t size    = (sizeof(memory)*((size_t)heap_size_percent))/100;

	if(!use_ebuddy)
	{
		return buddy_init(memory, size, minsize);
	}

	alloc_state = 0;

	return buddy_init_extendable(memory, size, minsize,
		2, 0, NULL, eb_alloc, eb_free);
}

int test_buddy2(const int use_ebuddy, const int shuffle_free, const int overwrite_allocated, const unsigned int large_div, const int heap_size_percent)
{
	struct buddy* const buddy = create_buddy(use_ebuddy, heap_size_percent);
	size_t small;
	size_t estim;
	size_t num;
	size_t large;
	void** ptrs;

#if DISPLAY_TITLE
	fprintf(stderr, "--------BUDDY2: %s, %s, %s, div=%u, %i%% of total\n",
		(use_ebuddy?"ebuddy":"sbuddy"),
		(shuffle_free?"shuffle":"sequential"),
		(overwrite_allocated?"overwrite":"no-fill"),
		large_div,
		heap_size_percent);
#endif

	if(NULL==buddy)
	{
		fprintf(stderr, "failed to init buddy\n");
		return 1;
	}

	small = buddy_get_smallest_size(buddy);
	estim = (buddy_get_total_free(buddy) / small) * (use_ebuddy ? 3 : 1);
	ptrs  = (void**)calloc(estim, sizeof(void*));

	if(NULL==ptrs)
	{
		fprintf(stderr, "out of system memory\n");
		return 1;
	}

	num = 0;

	while(0<(large=(buddy_get_largest_free(buddy)/large_div)))
	{
		void* const ptr = buddy_alloc(buddy, large);
#if DISPLAY_ALLOCATIONS
		fprintf(stderr, "%lu -> %p\n", (unsigned long)large, ptr);
#endif
		assert(NULL!=ptr);
		ptrs[num++] = ptr;

		if(overwrite_allocated)
		{
			memset(ptr, 0x7f, large);
		}
	}

	if(shuffle_free)
	{
		/* forge a seed in a deterministic way */
		srand(50 + use_ebuddy*100 + overwrite_allocated*200 + 400*heap_size_percent);
		shuffle(ptrs, num);
	}

	while(0<num)
	{
		--num;
		buddy_free(buddy, ptrs[num]);
	}

	buddy_deinit(buddy);
	free(ptrs);

	return 0;
}
